#!/usr/bin/perl
# cm@financial.com 07/2002
# mm@elabnet.de 20030901 completely changed to support v1/2
# mm@elabnet.de 20031220 fixed some handling for Net::SNMP < 5

use strict;
use Net::SNMP;
use Getopt::Std;

my %opts =(
        H => 'localhost',       # host 
        C => 'public',       # community
        d => 1,         # devicenumber
        w => 80,                # warnratio
        c => 90,                # critical ratio
		P => '2c',				# SNMP-version
        h => 0,					# help
        l => 0,					# list devices
        );

getopts('H:C:d:w:c:P:h:l:D',\%opts);

if ($opts{'h'} ) {
	# print "Usage: $0 [ -u <username> ] [ -l <snmp security level>] [ -a <snmp authentication protocol> ] [ -A <authentication protocol pass phrase> ] [ -x <snmp privacy protocol> ] [ -m <hostname>] [ -d <devicenumber> ] [ -w <warning ratio> ] [ -c <critical ratio ]\n";
	print "Usage: $0 -H <hostname> -d <devicenumber> [ -w <warning ratio> ] [ -c <critical ratio ] [ -C <community> -P <snmp-version|1|2> ] \n";
	exit 1;
}

#if ($opts{'w'} >= $opts{'c'}) {
#	print "Errorratio must be higher then Warnratio!\n";
#	exit 1;
#}

my ($session, $error) = Net::SNMP->session(
	-hostname	=>	$opts{'H'},
	-community	=>	$opts{'C'},
	-version	=>	$opts{'P'},
	);

if ($@) {
	print "SNMP-Error occured";
	exit 1;
}

if ( $opts{'l'} or $opts{'D'}) {
	my $result=undef;
	my $deviceNames=".1.3.6.1.2.1.25.2.3.1.3";
	my $OID=($deviceNames);
	print "Querying hrStorageTable on device..\n";
	$result = $session->get_table($OID);
	foreach my $oid (sort(keys(%{$result}))) {
			print "OID/Index $oid -- Description: $result->{$oid} \n";
	}
	exit 1 if ($opts{'l'});
}
my $result=undef;
my $message = "";
my $warnmsg = undef;
my $critmsg = undef;
my $diskidx;

my @devices = split(/,/,$opts{'d'}); 
my @warnlevels = split(/,/,$opts{'w'}); 
my @critlevels = split(/,/,$opts{'c'}); 

my $counter = 0;
foreach $diskidx (@devices) {
	$warnlevels[$counter] = 80 unless $warnlevels[$counter];
	$critlevels[$counter] = 90 unless $critlevels[$counter];
	if ($warnlevels[$counter] > $critlevels[$counter]) {
		print "ERROR: Disk #$diskidx WARN $warnlevels[$counter]% is higher than CRIT $critlevels[$counter]%\n";
		exit 1;
	}
	print "DEBUG: Query Disk #$diskidx WARNLEVEL: $warnlevels[$counter] CRITLEVEL: $critlevels[$counter] \n" if ($opts{'D'}); 
	getdiskspace($diskidx,$warnlevels[$counter],$critlevels[$counter]);
	$counter++;
}



sub getdiskspace {
my ($diskidx,$warn,$crit) = @_;
my @snmpoids;

my $deviceSize="1.3.6.1.2.1.25.2.3.1.5.$diskidx";
my $deviceUsed="1.3.6.1.2.1.25.2.3.1.6.$diskidx";
my $deviceName="1.3.6.1.2.1.25.2.3.1.3.$diskidx";
push(@snmpoids,$deviceSize);
push(@snmpoids,$deviceUsed);
push(@snmpoids,$deviceName);
$result = $session->get_request(@snmpoids);

if (!defined($result)) {
	print @snmpoids;
	printf("SNMP-ERROR: %s.\n", $session->error);
	$session->close;
	exit 1;
}

if (not $result->{$deviceSize} > 0) {
	print ("ERROR: Illegal values received for device #$diskidx Size: $result->{$deviceSize} Used: $result->{$deviceUsed}\n"); 
	$session->close;
	exit 1;
}

print ("DEBUG: Size: $result->{$deviceSize} Used: $result->{$deviceUsed}\n") if ($opts{'D'}); 
my $ratio = int($result->{$deviceUsed}*100/$result->{$deviceSize});
printf("DEBUG: %s usage %.2f%%\n", $result->{$deviceName}, $ratio) if ($opts{'D'}); 
# only get the first few chars from devicedescription to save space..
my ($devname,$dummy) = split("Serial",$result->{$deviceName},2);
$devname = substr($devname,0,18);
$devname =~ s/ *$//;

if ($ratio > $crit){
	$critmsg .= "$devname usage is $ratio% (CRITICAL) ! ";
} elsif ($ratio > $warn) {
	$warnmsg .= "$devname usage is $ratio% (WARNING) ! ";
} else {
	$message .= "$devname usage is $ratio% - OK (W$warn%/C$crit%) ";
}


} # end sub getdiskspace

$message = $critmsg . $warnmsg . $message;
if ($critmsg) {
	print "CRITICAL: $message\n";
	exit 2;
} 	
if ($warnmsg) {
	print "WARNING: $message\n";
	exit 1;
} 	

print "OK: $message\n";
exit 0;
